@@ -5,6 +5,7 @@ from __future__ import absolute_import
 import codecs
 import os
 import sys
+import warnings
 from os.path import join, abspath, dirname
 from collections import OrderedDict
 from arpeggio import DebugPrinter
@@ -16,6 +17,8 @@ from textx.const import MULT_ONE, MULT_ZEROORMORE, MULT_ONEORMORE, \
 from textx.exceptions import TextXError
 from .registration import LanguageDesc, metamodel_for_language
 from .model_params import ModelParams, ModelParamDefinitions
+from textx.scoping.rrel import create_rrel_scope_provider
+from textx.six import string_types
 
 if sys.version < '3':
     text = unicode  # noqa
@@ -37,10 +40,17 @@ def metamodel_from_str(lang_desc, metamodel=None, **kwargs):
 
     """
 
+    is_main_metamodel = metamodel is None
+
     if not metamodel:
         metamodel = TextXMetaModel(**kwargs)
 
-    language_from_str(lang_desc, metamodel)
+    file_name = kwargs.get('file_name')
+
+    language_from_str(lang_desc, metamodel, file_name)
+
+    if is_main_metamodel:
+        metamodel.validate_user_classes()
 
     return metamodel
 
@@ -121,8 +131,13 @@ class TextXMetaModel(DebugPrinter):
         builtins(dict): A dict of named object used in linking phase.
             References to named objects not defined in the model will be
             searched here.
-        classes(list of classes): A list of user supplied classes to use
-            instead of the dynamically created.
+        builtin_models(ModelRepository): An optional repository of preloaded models
+            to be used during reference resolving. These models will be searched
+            as a fallback before `builtins`.
+        classes(list of classes or callable): A list of user supplied classes
+            to use instead of the dynamically created or a callable providing
+            those classes. The callable must accept a rule name and return a
+            class for that rule name or None.
         obj_processors(dict): A dict of user supplied object processors keyed
             by rule/class name (may be a fully qualified name).
         rootcls(TextXClass): A language class that is a root of the meta-model.
@@ -151,9 +166,10 @@ class TextXMetaModel(DebugPrinter):
     """
 
     def __init__(self, file_name=None, classes=None, builtins=None,
-                 auto_init_attributes=True, ignore_case=False, skipws=True,
-                 ws=None, autokwd=False, memoization=False,
-                 textx_tools_support=False, use_regexp_group=False, **kwargs):
+                 builtin_models=None, auto_init_attributes=True,
+                 ignore_case=False, skipws=True, ws=None, autokwd=False,
+                 memoization=False, textx_tools_support=False,
+                 use_regexp_group=False, **kwargs):
         # evaluate optional parameter "global_repository"
         global_repository = kwargs.pop("global_repository", False)
         if global_repository:
@@ -165,20 +181,25 @@ class TextXMetaModel(DebugPrinter):
 
         super(TextXMetaModel, self).__init__(**kwargs)
 
-        self._tx_model_param_definitions = ModelParamDefinitions()
-        self._tx_model_param_definitions.add(
-            "project_root", "the project root path")
+        self.model_param_defs = ModelParamDefinitions()
+        self.model_param_defs.add("project_root", "the project root path")
 
         self.file_name = file_name
         self.rootcls = None
 
         self.builtins = builtins
+        self.builtin_models = builtin_models
 
         # Convert classes to dict for easier lookup
         self.user_classes = {}
+        self.user_classes_provider = None
+        self._used_rule_names_for_user_classes = set()
         if classes:
-            for c in classes:
-                self.user_classes[c.__name__] = c
+            if callable(classes):
+                self.user_classes_provider = classes
+            else:
+                for c in classes:
+                    self.user_classes[c.__name__] = c
 
         self.auto_init_attributes = auto_init_attributes
         self.ignore_case = ignore_case
@@ -254,6 +275,9 @@ class TextXMetaModel(DebugPrinter):
 
     def register_scope_providers(self, sp):
         self.scope_providers = sp
+        for k, v in self.scope_providers.items():
+            if isinstance(v, string_types):
+                self.scope_providers[k] = create_rrel_scope_provider(v)
 
     def _namespace_for_file_name(self, file_name):
         if file_name is None or self.root_path is None:
@@ -486,6 +510,21 @@ class TextXMetaModel(DebugPrinter):
         textX rules.
         """
         # TODO: Implement complex textX validations.
+        pass
+
+    def validate_user_classes(self):
+        """
+        Validates user classes of the meta model.
+        Called after construction of the main metamodel (not
+        imported ones).
+        """
+        from textx.exceptions import TextXSemanticError
+        for user_class in self.user_classes.values():
+            if user_class.__name__ not in self._used_rule_names_for_user_classes:
+                # It is not a user class used in the grammar
+                raise TextXSemanticError(
+                    "{} class is not used in the grammar".format(
+                        user_class.__name__))
 
     def __getitem__(self, name):
         """
@@ -561,7 +600,7 @@ class TextXMetaModel(DebugPrinter):
                 is set while executing pre_ref_resolution_callback (see
                 scoping.md)
         """
-        self._tx_model_param_definitions.check_params('from_str', **kwargs)
+        self.model_param_defs.check_params('from_str', **kwargs)
 
         if type(model_str) is not text:
             raise TextXError("textX accepts only unicode strings.")
@@ -590,7 +629,7 @@ class TextXMetaModel(DebugPrinter):
 
     def model_from_file(self, file_name, encoding='utf-8', debug=None,
                         **kwargs):
-        self._tx_model_param_definitions.check_params(file_name, **kwargs)
+        self.model_param_defs.check_params(file_name, **kwargs)
 
         return self.internal_model_from_file(
             file_name, encoding, debug,
@@ -622,13 +661,11 @@ class TextXMetaModel(DebugPrinter):
                     # print("METAMODEL PRE-CALLBACK => {}".format(filename))
                     other_model._tx_model_repository = GlobalModelRepository(
                         self._tx_model_repository.all_models)
-                    self._tx_model_repository.all_models\
-                        .filename_to_model[filename] = other_model
+                    self._tx_model_repository.all_models[filename] = other_model
 
                 callback = _pre_ref_resolution_callback
             if self._tx_model_repository.all_models.has_model(file_name):
-                model = self._tx_model_repository.all_models\
-                    .filename_to_model[file_name]
+                model = self._tx_model_repository.all_models[file_name]
 
         def kwargs_callback(other_model):
             if hasattr(other_model, '_tx_metamodel'):
@@ -674,6 +711,12 @@ class TextXMetaModel(DebugPrinter):
         self.obj_processors = obj_processors
         self.type_convertors.update(obj_processors)
 
+    @property
+    def _tx_model_param_definitions(self):
+        warnings.warn(
+            '_tx_model_param_definitions is deprecated in favor of model_param_defs.')
+        return self.model_param_defs
+
 
 class TextXMetaMetaModel(object):
     """
