@@ -25,6 +25,31 @@ __all__ = ['get_children_of_type', 'get_parent_of_type', 'get_model',
            'get_metamodel']
 
 
+def textx_isinstance(obj, obj_cls):
+    """
+    This function determines, if a textx object is an instance of a
+     textx class.
+    Args:
+        obj: the object to be analyzed
+        obj_cls: the class to be checked
+
+    Returns:
+        True if obj is an instance of obj_cls.
+    """
+    if obj_cls.__name__ == "OBJECT":
+        return True
+    if isinstance(obj, obj_cls):
+        return True
+    if hasattr(obj_cls, "_tx_fqn") and hasattr(obj, "_tx_fqn"):
+        if obj_cls._tx_fqn == obj._tx_fqn:
+            return True
+    if hasattr(obj_cls, "_tx_inh_by"):
+        for cls in obj_cls._tx_inh_by:
+            if (textx_isinstance(obj, cls)):
+                return True
+    return False
+
+
 def get_model(obj):
     """
     Finds model root element for the given object.
@@ -62,18 +87,20 @@ def get_parent_of_type(typ, obj):
             return obj
 
 
-def get_children(decider, root, children_first=False):
+def get_children(selector, root, children_first=False, should_follow=lambda obj: True):
     """
     Returns a list of all model elements of type 'typ' starting from model
     element 'root'. The search process will follow containment links only.
     Non-containing references shall not be followed.
 
     Args:
-        decider(obj): a callable returning True if the object is of interest.
+        selector(callable): a predicate to decide if the object is of interest.
         root (model object): Python model object which is the start of the
             search process.
         children_first (bool): a flag indicating whether children will be
             returned before their parents (default=False)
+        should_follow(callable): A predicate to decide if the element should be
+            traversed.
     """
     collected = []
     collected_ids = set()
@@ -88,7 +115,7 @@ def get_children(decider, root, children_first=False):
         cls = elem.__class__
 
         if not children_first:
-            if hasattr(cls, '_tx_attrs') and decider(elem):
+            if hasattr(cls, '_tx_attrs') and selector(elem):
                 collected.append(elem)
                 collected_ids.add(id(elem))
 
@@ -98,16 +125,17 @@ def get_children(decider, root, children_first=False):
                 if attr.cont:
                     if attr.mult in (MULT_ONE, MULT_OPTIONAL):
                         new_elem = getattr(elem, attr_name)
-                        if new_elem:
+                        if new_elem is not None and should_follow(new_elem):
                             follow(new_elem)
                     else:
                         new_elem_list = getattr(elem, attr_name)
                         if new_elem_list:
                             for new_elem in new_elem_list:
-                                follow(new_elem)
+                                if should_follow(new_elem):
+                                    follow(new_elem)
 
         if children_first:
-            if hasattr(cls, '_tx_attrs') and decider(elem):
+            if hasattr(cls, '_tx_attrs') and selector(elem):
                 collected.append(elem)
                 collected_ids.add(id(elem))
 
@@ -115,7 +143,7 @@ def get_children(decider, root, children_first=False):
     return collected
 
 
-def get_children_of_type(typ, root):
+def get_children_of_type(typ, root, children_first=False, should_follow=lambda obj: True):
     """
     Returns a list of all model elements of type 'typ' starting from model
     element 'root'. The search process will follow containment links only.
@@ -126,12 +154,35 @@ def get_children_of_type(typ, root):
             looking for.
         root (model object): Python model object which is the start of the
             search process.
+        children_first (bool): a flag indicating whether children will be
+            returned before their parents (default=False)
+        should_follow(callable): A predicate to decide if the element should be
+            traversed.
     """
 
     if type(typ) is not text:
         typ = typ.__name__
 
-    return get_children(lambda x: x.__class__.__name__ == typ, root)
+    return get_children(lambda x: x.__class__.__name__ == typ, root,
+                        children_first=children_first,
+                        should_follow=should_follow)
+
+
+def get_location(model_obj):
+    """
+    Args:
+        model_obj: the model object of interest
+
+    Returns:
+        the line, col and filename of the model element.
+        The filename may be None.
+        This function may be used to fill exceptions
+    """
+    from textx.model import get_model
+    the_model = get_model(model_obj)
+    line, col = the_model._tx_parser.pos_to_linecol(
+        model_obj._tx_position)
+    return {"line": line, "col": col, "filename": the_model._tx_filename}
 
 
 class ObjCrossRef(object):
@@ -142,12 +193,20 @@ class ObjCrossRef(object):
         obj_name(str): A name of the target object.
         cls(TextXClass): The target object class.
         position(int): A position in the input string of this cross-ref.
+        scope_provider(scope provider): A scope provider for that
+            reference (see scoping.md for requirements of a scope provider)
+        match_rule_name: the rule name which defines the text format of
+            the reference. It is required to extract the split-attribute for
+            locally defined scope providers.
     """
 
-    def __init__(self, obj_name, cls, position):
+    def __init__(self, obj_name, cls, position, scope_provider,
+                 match_rule_name):
         self.obj_name = obj_name
         self.cls = cls
         self.position = position
+        self.scope_provider = scope_provider
+        self.match_rule_name = match_rule_name
 
 
 class RefRulePosition(object):
@@ -297,6 +356,8 @@ def get_model_parser(top_rule, comments_model, **kwargs):
             return model
 
         def _replace_user_attr_methods_for_class(self, user_class):
+            assert hasattr(user_class, "_tx_obj_attrs")
+
             # Custom attr dunder methods used for user classes during loading
             def _getattribute(obj, name):
                 if name == '__dict__':
@@ -314,19 +375,19 @@ def get_model_parser(top_rule, comments_model, **kwargs):
 
             def _setattr(obj, name, value):
                 try:
-                    obj._tx_obj_attrs[id(obj)][name] = value
+                    user_class._tx_obj_attrs[id(obj)][name] = value
                 except KeyError:
                     try:
-                        return obj._tx_real_setattr(name, value)
+                        return user_class._tx_real_setattr(name, value)
                     except (AttributeError, TypeError):
                         return super(user_class, obj).__setattr__(name, value)
 
             def _delattr(obj, name):
                 try:
-                    obj._tx_obj_attrs[id(obj)].pop(name)
+                    user_class._tx_obj_attrs[id(obj)].pop(name)
                 except KeyError:
                     try:
-                        return obj._tx_real_delattr(name)
+                        return user_class._tx_real_delattr(name)
                     except (AttributeError, TypeError):
                         return super(user_class, obj).__delattr__(name)
 
@@ -509,7 +570,15 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
                 # Objects of each class are in its own namespace
                 if not id(inst.__class__) in parser._instances:
                     parser._instances[id(inst.__class__)] = {}
-                parser._instances[id(inst.__class__)][inst.name] = inst
+                try:
+                    parser._instances[id(inst.__class__)][inst.name] = inst
+                except TypeError as e:
+                    if 'unhashable type' in e.args[0]:
+                        raise TextXSemanticError(
+                            'Object name can\'t be of unhashable type.'
+                            ' Please see the note in this docs'
+                            ' section http://textx.github.io/textX/stable/grammar/#references')  # noqa
+                    raise
 
             if parser.debug:
                 parser.dprint("LEAVING INSTANCE {}".format(node.rule_name))
@@ -543,8 +612,12 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
 
                 if metaattr.ref and not metaattr.cont:
                     # If this is non-containing reference create ObjCrossRef
+                    p = metaattr.scope_provider
+                    rn = metaattr.match_rule_name
                     value = ObjCrossRef(obj_name=value, cls=metaattr.cls,
-                                        position=node[0].position)
+                                        position=node[0].position,
+                                        scope_provider=p,
+                                        match_rule_name=rn)
                     parser._crossrefs.append((model_obj, metaattr, value))
                     return model_obj
 
@@ -565,9 +638,13 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
                             # If this is non-containing reference
                             # create ObjCrossRef
 
+                            p = metaattr.scope_provider
+                            rn = metaattr.match_rule_name
                             value = ObjCrossRef(obj_name=value,
                                                 cls=metaattr.cls,
-                                                position=n.position)
+                                                position=n.position,
+                                                scope_provider=p,
+                                                match_rule_name=rn)
 
                             parser._crossrefs.append((obj_attr, metaattr,
                                                       value))
@@ -707,6 +784,14 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
             if isinstance(scope_provider, ModelLoader):
                 scope_provider.load_models(model, encoding=encoding)
 
+        for crossref in parser._crossrefs:
+            crossref = crossref[2]
+            if crossref.scope_provider is not None:
+                from textx.scoping import ModelLoader
+                scope_provider = crossref.scope_provider
+                if isinstance(scope_provider, ModelLoader):
+                    scope_provider.load_models(model, encoding=encoding)
+
         if not is_immutable_obj:
             model._tx_reference_resolver = ReferenceResolver(
                 parser, model, pos_crossref_list)
@@ -922,15 +1007,18 @@ class ReferenceResolver:
                 attr_refs = [obj.__class__.__name__ + "." + attr.name,
                              "*." + attr.name, obj.__class__.__name__ + ".*",
                              "*.*"]
-                for attr_ref in attr_refs:
-                    if attr_ref in metamodel.scope_providers:
-                        if self.parser.debug:
-                            self.parser.dprint(" FOUND {}".format(attr_ref))
-                        resolved = metamodel.scope_providers[attr_ref](
-                            obj, attr, crossref)
-                        break
+                if crossref.scope_provider is not None:
+                    resolved = crossref.scope_provider(obj, attr, crossref)
                 else:
-                    resolved = default_scope(obj, attr, crossref)
+                    for attr_ref in attr_refs:
+                        if attr_ref in metamodel.scope_providers:
+                            if self.parser.debug:
+                                self.parser.dprint(" FOUND {}".format(attr_ref))
+                            resolved = metamodel.scope_providers[attr_ref](
+                                obj, attr, crossref)
+                            break
+                    else:
+                        resolved = default_scope(obj, attr, crossref)
 
                 # Collect cross-references for textx-tools
                 if resolved is not None and not type(resolved) is Postponed:
